/*
 * Copyright (c) 2019-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: BSD-3-Clause
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include "array/Array1D.h"

namespace visrtx {

struct ObjectArray : public Array
{
  ObjectArray(void *appMemory,
      ANARIMemoryDeleter deleter,
      void *deleterPtr,
      ANARIDataType type,
      uint64_t numItems,
      uint64_t byteStride);
  ~ObjectArray();

  ArrayShape shape() const override;

  size_t totalSize() const override;

  size_t size() const;

  void privatize() override;

  Object **handles();

  void *deviceData() const override;

  void appendHandle(Object *);
  void removeAppendedHandles();

  void uploadArrayData() const override;

 private:
  void updateInternalHandleArray();

  std::vector<Object *> m_handleArray;
  mutable thrust::host_vector<void *> m_GPUDataHost;
  mutable thrust::device_vector<void *> m_GPUDataDevice;
  size_t m_numAppended{0};
};

// Object specializations /////////////////////////////////////////////////////

template <>
inline ObjectArray *Object::getParamObject<ObjectArray>(
    const std::string &name, ObjectArray *valIfNotFound)
{
  if (!hasParam(name))
    return valIfNotFound;

  using PTR_T = anari::IntrusivePtr<Array1D>;
  PTR_T val = getParam<PTR_T>(name, PTR_T());
  return (ObjectArray *)val.ptr;
}

} // namespace visrtx

VISRTX_ANARI_TYPEFOR_SPECIALIZATION(visrtx::ObjectArray *, ANARI_ARRAY1D);
